@@ -32,6 +32,7 @@ from dbus_fast import BusType, Message, MessageType
 from proton.vpn.cli.commands.account import signin, signout, info
 from proton.vpn.cli.commands.server import connect, disconnect
 from proton.vpn.cli.commands.location_discovery import countries, cities
+from proton.vpn.cli.commands.set import config
 from proton.vpn.cli.core.controller import Params
 
 try:
@@ -124,6 +125,9 @@ app.add_command(disconnect)
 app.add_command(countries)
 app.add_command(cities)
 
+# set features
+app.add_command(config)
+
 
 def main():
     """Runs the CLI."""
