@@ -32,13 +32,16 @@ from packaging.version import Version
 import sentry_sdk
 
 from proton.session.exceptions import ProtonAPIAuthenticationNeeded
+from proton.vpn.core.settings.custom_dns import CustomDNSEntry, CustomDNS
 from proton.vpn import logging as ProtonLogging
 from proton.vpn.cli.core.exception_handler import ExceptionHandler
 from proton.vpn.cli.core.exceptions import \
     AuthenticationRequiredError, \
     CountryCodeError, \
     CountryNameError, \
-    RequiresHigherTierError
+    RequiresHigherTierError, \
+    InvalidDNS, \
+    VPNConnectionError
 from proton.vpn.connection import states
 from proton.vpn.connection.enum import ConnectionStateEnum
 from proton.vpn.core.api import ProtonVPNAPI
@@ -55,18 +58,24 @@ LOGGING_FILENAME = "vpn-cli"
 DEFAULT_CLI_NAME = "protonvpn"
 
 
+@dataclass
+class Feature:
+    """Used when setting and saving features.
+    """
+    command: str
+    human_friendly_name: str
+    setting_path: str
+    short_help: str = None
+    available_on_free_tier: bool = False
+    requires_restart: bool = False
+
+
 @dataclass
 class Params:
     """The parameters for constructing the Controller"""
     verbose: str = False
 
 
-class VPNConnectionError(Exception):
-    """
-    Error establishing a VPN server connection
-    """
-
-
 @asynccontextmanager
 async def _wait_for_event(  # pylint: disable=R0913
     connector: VPNConnector,
@@ -114,7 +123,7 @@ async def _wait_for_event(  # pylint: disable=R0913
         raise VPNConnectionError
 
 
-class Controller:
+class Controller:  # pylint: disable=too-many-public-methods
     """
     The application business logic is in this class. The is the core of the
     application.
@@ -144,7 +153,10 @@ class Controller:
     async def create(params: Params, click_ctx: ClickContext):
         """Preferred method to get an instance of Controller."""
         controller = Controller(params, click_ctx)
-        await controller.get_settings()  # load settings
+        # Ensure controller always has settings loaded,
+        # even if only using free user defaults prior to authentication.
+        # This allows crash reporting to work prior to sign in.
+        await controller.get_settings()
         return controller
 
     def set_uncaught_exceptions_to_absorb(self, exceptions: List[BaseException]):
@@ -188,10 +200,96 @@ class Controller:
         """Returns the Proton VPN tier"""
         return self._api.user_tier
 
+    @property
+    def user_on_free_tier(self) -> bool:
+        """Returns if the current user is on free tier or not.
+        """
+        return self.user_tier == 0
+
     async def get_settings(self) -> Settings:
         """Returns general settings."""
         return await self._api.load_settings()
 
+    async def save_config(
+        self,
+        feature: Feature,
+        value: Union[int, bool, CustomDNS]
+    ):
+        """Ensures that the feature is stored to disk only
+        if the subscription tier allows is.
+
+        Args:
+            settings (Settings): settings object with the modified feature
+            feature (Feature): feature object that is used for the check
+
+        Raises:
+            AuthenticationRequiredError: if user is not logged in
+            RequiresHigherTierError: if feature requires a higher plan
+        """
+        if not self.is_logged_in:
+            raise AuthenticationRequiredError
+
+        settings = await self.get_settings()
+
+        if not feature.available_on_free_tier and self.user_on_free_tier:
+            raise RequiresHigherTierError
+
+        settings = self._set_settings_by_path(settings, feature.setting_path, value)
+
+        await self.save_settings(settings)
+
+    async def save_settings(self, settings: Settings):
+        """Returns general settings."""
+        # We need to call get_vpn_connector() because the API
+        # applies settings to current connection, even though
+        # currently it seems like it's not working.
+        await self.get_vpn_connector()
+        await self._api.save_settings(settings)
+
+    def _set_settings_by_path(
+        self, settings: Settings, path: str,
+        value: Union[int, bool, CustomDNS]
+    ):
+        parts = path.split(".")
+        current = settings
+        for part in parts[:-1]:
+            current = getattr(current, part)
+
+        setattr(current, parts[-1], value)
+        return settings
+
+    def parse_dns_ips(self, dns_list: list[str]) -> list[CustomDNSEntry]:
+        """Parses a CSV string of DNS IPs into a list of strings.
+
+        Args:
+            dns_csv (str): CSV string of DNS IPs.
+
+        Returns:
+            List[str]: List of DNS IPs.
+
+        Raises:
+            ValueError: If any of the DNS IPs are invalid.
+        """
+        parsed_dns_list = []
+        for dns in dns_list:
+            try:
+                parsed_dns_list.append(CustomDNSEntry.new_from_string(dns))
+            except ValueError as excp:
+                raise InvalidDNS(dns, excp) from excp
+        return parsed_dns_list
+
+    def to_custom_dns(self, state: bool, dns_ips: list[CustomDNSEntry]) -> CustomDNS:
+        """Convert to custom dns setting object.
+
+        Args:
+            state (bool): whether is enabled or disabled
+            dns_ips (list[CustomDNSEntry]): list of IPs
+
+        Returns:
+            CustomDNS: created object
+        """
+        return CustomDNS(state, dns_ips)
+
     async def get_current_connection(self) -> Optional[VPNConnection]:
         """Returns the current VPN connection or None if there isn't one."""
         return (await self.get_vpn_connector()).current_connection
